@@ -10,29 +10,32 @@ using namespace std;
 int main (int argc, char **argv) {
   environment rheolef (argc,argv);
   geo omega (argv[1]);
-  Float  nu     = (argc > 2) ? atof(argv[2]) : 1e-2;
-  size_t n_max  = (argc > 3) ? atoi(argv[3]) : 50;
+  string approx = (argc > 2) ? argv[2] : "P1";
+  Float  nu     = (argc > 3) ? atof(argv[3]) : 1e-2;
+  size_t n_max  = (argc > 4) ? atoi(argv[4]) : 50;
   size_t d = omega.dimension();
   Float delta_t = 2*acos(-1.)/n_max;
-  space Vh (omega, "P1", "vector");
+  space Vh (omega, approx, "vector");
   field uh = interpolate (Vh, u(d));
-  space Xh (omega, "P1");
+  space Xh (omega, approx);
   Xh.block ("boundary");
   field phi_h      = interpolate (Xh, phi(d,nu,0));
   field phi_h_prec = interpolate (Xh, phi(d,nu,-delta_t));
   characteristic X1 (  -delta_t*uh);
   characteristic X2 (-2*delta_t*uh);
-  form m (Xh, Xh, "lumped_mass");
-  form a (Xh, Xh, "grad_grad");
+  quadrature_option_type qopt;
+  qopt.set_family (quadrature_option_type::gauss_lobatto);
+  qopt.set_order (Xh.degree());
+  trial varphi (Xh); test psi (Xh);
   branch event ("t","phi");
   dout << catchmark("nu") << nu << endl
        << event (0, phi_h);
   for (size_t n = 1; n <= n_max; n++) {
     Float t = n*delta_t;
-    field l1h = riesz(Xh, compose(phi_h, X1));
-    field l2h = riesz(Xh, compose(phi_h_prec, X2));
-    field lh = 2*l1h - 0.5*l2h;
-    form c = (1.5 + delta_t*phi::sigma(d,nu,t))*m + delta_t*nu*a;
+    Float alpha = 1.5 + delta_t*phi::sigma(d,nu,t);
+    Float beta  = delta_t*nu;
+    form c = integrate (alpha*varphi*psi + beta*dot(grad(varphi),grad(psi)), qopt);
+    field lh = integrate ((2.*compose(phi_h, X1) - 0.5*compose(phi_h_prec, X2))*psi, qopt);
     solver sc (c.uu());
     phi_h_prec = phi_h;
     phi_h.set_u() = sc.solve (lh.u() - c.ub()*phi_h.b());
