@@ -10,8 +10,9 @@ int main(int argc, char**argv) {
   size_t d = Xh.get_geo().dimension();
   field pi_h_u = interpolate(Xh, u(d));
   field eh = uh - pi_h_u;
-  form m(Xh, Xh, "mass");
-  form a(Xh, Xh, "grad_grad");
+  trial u (Xh); test v (Xh);
+  form m = integrate (u*v);
+  form a = integrate (dot(grad(u),grad(v)));
   dout << "error_l2   " << sqrt(m(eh,eh)) << endl
        << "error_linf " << eh.max_abs()   << endl
        << "error_h1   " << sqrt(a(eh,eh)) << endl;
