@@ -8,17 +8,15 @@ int main (int argc, char **argv) {
   Float delta_t = 0.5/n_max;
   space Xh (omega, "P1");
   Xh.block ("boundary");
-  form m (Xh, Xh, "mass");
-  form a (Xh, Xh, "grad_grad");
-  form c = m + delta_t*a;
-  solver sc = ldlt (c.uu());
-  field lh = riesz (Xh, 1);
+  trial u (Xh); test v (Xh);
+  form a = integrate (u*v + delta_t*dot(grad(u),grad(v)));
+  solver sa = ldlt (a.uu());
   field uh (Xh, 0);
   branch event ("t","u");
   dout << event (0, uh);
   for (size_t n = 1; n <= n_max; n++) {
-    field kh = m*uh + delta_t*lh;
-    uh.set_u() = sc.solve (kh.u() - c.ub()*uh.b());
-    dout << event (Float(n)*delta_t, uh);
+    field lh = integrate ((uh+delta_t)*v);
+    uh.set_u() = sa.solve (lh.u() - a.ub()*uh.b());
+    dout << event (n*delta_t, uh);
   }
 }
