@@ -8,10 +8,11 @@ int main (int argc, char**argv) {
   field uh;
   din >> uh;
   const space& Wh = uh.get_space();
-  field pi_h_u = interpolate(Wh, u());
+  field pi_h_u = interpolate(Wh, u_exact());
   field eh = uh-pi_h_u;
-  form m (Wh, Wh, "mass");
-  form a (Wh, Wh, "grad_grad");
+  trial u (Wh); test v (Wh);
+  form m = integrate (u*v);
+  form a = integrate (dot(grad_s(u),grad_s(v)));
   field one (Wh, 1);
   Float meas_gamma = m(one,one);
   Float lambda_e = m(one,eh)/meas_gamma;
