@@ -52,6 +52,8 @@ class point_basic {
 // typedefs:
 
 	typedef size_t size_type;
+	typedef T      element_type;
+	typedef T      scalar_type;
 	typedef T      float_type;
 
 // allocators:
@@ -130,10 +132,10 @@ class point_basic {
 	point_basic<T> operator- (const point_basic<T>& v) const
 		{ return point_basic<T> (_x[0]-v[0], _x[1]-v[1], _x[2]-v[2]); }
 
-	point_basic<T> operator* (T a) const
+	point_basic<T> operator* (const T& a) const
 		{ return point_basic<T> (a*_x[0], a*_x[1], a*_x[2]); }
 	point_basic<T> operator* (int a) const
-		{ return operator* (a); }
+		{ return operator* (T(a)); }
 
 	point_basic<T> operator/ (const T& a) const
 		{ return operator* (T(1)/T(a)); }
@@ -155,7 +157,7 @@ inline
 point_basic<T>
 operator* (int a, const point_basic<T>& u)
 {
-  return u.operator* (a);
+  return u.operator* (T(a));
 }
 template<class T>
 inline
@@ -270,6 +272,11 @@ lexicographically_less (const point_basic<T>& a, const point_basic<T>& b)
 // -------------------------------------------------------------------------------------
 // inline'd
 // -------------------------------------------------------------------------------------
+/// @brief helper for point_basic<T> & tensor_basic<T>: get basic T type
+template<class T> struct scalar_traits                  { typedef T type; };
+template<class T> struct scalar_traits<point_basic<T> > { typedef T type; };
+template<class T> struct  float_traits<point_basic<T> > { typedef typename float_traits<T>::type type; };
+
 // input/output
 template<class T>
 inline
