@@ -43,6 +43,7 @@ public:
 
   typedef size_t size_type;
   typedef T      element_type;
+  typedef T      float_type;
 
 // allocators:
 
@@ -68,6 +69,9 @@ typedef tensor4_basic<Float> tensor4;
 // -----------------------------------------------------------------------
 // inlined
 // -----------------------------------------------------------------------
+template<class T> struct  float_traits<tensor4_basic<T> > { typedef typename float_traits<T>::type type; };
+template<class T> struct scalar_traits<tensor4_basic<T> > { typedef T type; };
+
 template<class T>
 inline
 tensor4_basic<T>&
