@@ -40,15 +40,15 @@ EXAMPLE:
 INPUT FILE SPECIFICATION:
     @table @code
 
-    @itemx @var{filename}
+    @item @var{filename}
 	specifies the name of the file containing
       	the input field.
 
-    @itemx -
+    @item -
 	read field on standard input instead on a file.
 
 @cindex RHEOPATH environment variable
-    @itemx -I@var{dir} 
+    @item -I@var{dir} 
     @itemx -I @var{dir} 
         Add @var{dir} to the rheolef file search path.
 	This option is usefull e.g. when the mesh .geo and the .field fikes are in
@@ -65,20 +65,20 @@ INPUT FILE SPECIFICATION:
 
 OUTPUT FILE FORMAT OPTIONS:
   @table @code
-  @itemx -field
+  @item -field
 	output field on standard output stream in field text file format.
 @toindex @code{gmsh}
 @fiindex @file{.gmsh} mesh file
 @fiindex @file{.gmsh_pos} data file
-  @itemx -gmsh
+  @item -gmsh
 	output field on standard output stream in gmsh text file format.
-  @itemx -gmsh-pos
+  @item -gmsh-pos
 	output field on standard output stream in gmsh-pos text file format,
         suitable for mesh adaptation purpose.
 @toindex @code{bamg}
 @fiindex @file{.bamg} mesh file
 @fiindex @file{.bamg_bb} data file
-  @itemx -bamg-bb
+  @item -bamg-bb
 	output field on standard output stream in bamg-bb text file format,
         suitable for mesh adaptation purpose.
 @cindex graphic render
@@ -89,7 +89,7 @@ OUTPUT FILE FORMAT OPTIONS:
 @fiindex @file{.jpg} image
 @fiindex @file{.ps} image
 @fiindex @file{.pdf} image
-  @itemx -image-format @code{string}
+  @item -image-format @code{string}
         The argument is any valid image format, such as
         bitmap @code{png}, @code{jpg}, @code{gif} or vectorial @code{ps} or @code{pdf}
         image file formats, and that could be handled by the corresponding render.
@@ -100,7 +100,7 @@ OUTPUT FILE FORMAT OPTIONS:
 
 GETTING INFORMATION:
     @table @code
-    @itemx -min
+    @item -min
     @itemx -max
         print the min (resp. max) value of the scalar field and then exit.
     @end table
@@ -109,66 +109,66 @@ RENDER OPTIONS:
   @table @code
 
 @toindex @code{gnuplot}
-  @itemx -gnuplot
+  @item -gnuplot
 	use @code{gnuplot} tool.
 	This is the default in one and two dimension.
 @toindex @code{mayavi}
-  @itemx -mayavi
+  @item -mayavi
         use @code{mayavi} tool.
         This is the default for tridimensional geometries.
   @end table
 
 RENDERING OPTIONS:
     @table @code
-    @itemx -color
+    @item -color
     @itemx -gray
     @itemx -black-and-white
     @itemx -bw
 	Use (color/gray scale/black and white) rendering.
 	Color rendering is the default.
-    @itemx -elevation
+    @item -elevation
     @itemx -noelevation
 	For two dimensional field, represent values as elevation
 	in the third dimension.
 	The default is no evelation.
-    @itemx -scale @var{float}
+    @item -scale @var{float}
         applies a multiplicative factor to the field.
         This is useful e.g. in conjonction with the @code{elevation} option.
         The default value is 1.
-    @itemx -stereo
+    @item -stereo
     @itemx -nostereo
         Rendering mode suitable for red-blue anaglyph 3D stereoscopic glasses.
         This option is only available with @code{mayavi}.
-    @itemx -fill
+    @item -fill
 	isoline intervals are filled with color.
 	This is the default.
-    @itemx -nofill
+    @item -nofill
 	draw isolines by using lines.
-    @itemx -cut
+    @item -cut
     @itemx -nocut
         Cut by a specified plane.
         The cutting plane is specified by its origin point and normal vector.
         This option requires the @code{mayavi} code.
-    @itemx -origin @var{float} [@var{float} [@var{float}]]
+    @item -origin @var{float} [@var{float} [@var{float}]]
         set the origin of the cutting plane.
         Default is (0.5, 0.5, 0.5).
-    @itemx -normal @var{float} [@var{float} [@var{float}]]
+    @item -normal @var{float} [@var{float} [@var{float}]]
         set the normal of the cutting plane.
         Default is (1, 0, 0).
-    @itemx -iso [@var{float}]
+    @item -iso [@var{float}]
         do draw 3d isosurface. When the optional float is not provided,
 	a median value is used.
         This option requires the @code{mayavi} code.
-    @itemx -noiso
+    @item -noiso
         do not draw isosurface.
-    @itemx -n-iso @var{int}
+    @item -n-iso @var{int}
         For 2D visualizations, the isovalue table contains
         regularly spaced values from fmin to fmax, the bounds
         of the field.
 @cindex projection
-    @itemx -proj
+    @item -proj
         Convert all selected fields to Pk-continuous approximation by using a L2 projection.
-    @itemx -round [@var{float}]
+    @item -round [@var{float}]
         Round the input up to the specifiexd precision.
         This option, combined with @code{-field}, leads to a round filter.
         Usefull for non-regression test purpose, in order
@@ -177,7 +177,7 @@ RENDERING OPTIONS:
 
 @cindex vorticity
 @cindex stream function
-    @itemx -n-iso-negative @var{int}
+    @item -n-iso-negative @var{int}
         The isovalue table is splitted into negatives and positives values.
         Assume there is n_iso=15 isolines: if 4 is requested by this option,
         then, there will be 4 negatives isolines, regularly spaced
@@ -185,44 +185,48 @@ RENDERING OPTIONS:
         from 0 to fmax.
         This option is usefull when plotting e.g. vorticity or stream
         functions, where the sign of the field is representative.
-    @itemx -subdivide int
+    @item -subdivide int
         When using a high order geometry, the number of points per edge used to draw
         a curved element. Default value is the mesh order.
 
-    @itemx -deformation
+    @item -deformation
         Render vector-valued fields as deformed mesh
         using @code{mayavi} or @code{gnuplot}.
         This is the default vector field representation.
 
-    @itemx -velocity
+    @item -velocity
         Render vector-valued fields as arrows using @code{mayavi}.
 
     @end table
-COMPONENT EXTRACTION:
+COMPONENT EXTRACTION AND DOMAIN REDUCTION:
     @table @code
-    @itemx -comp @var{int}
-	Extract a component of a vector-valued field.
+    @item -comp @var{int}
+    @itemx -comp @var{string}
+	Extract the i-th component of a vector-valued field.
+        For a tensor-valued field, indexing components as "00", "01", "11"... is supported.
+    @item -domain @var{name}
+	Reduce the visualization to the specified domain.
     @end table
 
 OTHERS OPTIONS:
     @table @code
-    @itemx -verbose
+    @item -verbose
 	print messages related to graphic files created and
        command system calls (this is the default).
 
-    @itemx -noverbose
+    @item -noverbose
 	does not print previous messages.
 
-    @itemx -clean
+    @item -clean
 	clear temporary graphic files (this is the default).
     
-    @itemx -noclean
+    @item -noclean
 	does not clear temporary graphic files.
 
-    @itemx -execute
+    @item -execute
 	execute graphic command (this is the default).
     
-    @itemx -noexecute
+    @item -noexecute
 	does not execute graphic command. Generates only 
 	graphic files. This is usefull in conjuction with the
 	@code{-noclean} command.
@@ -305,7 +309,8 @@ void usage() {
            << "[-n-iso-negative int] "
            << "[-image-format string] "
            << "[-subdivide int] "
-           << "[-comp int] "
+           << "[-comp string] "
+           << "[-domain string] "
            << "[-proj] "
            << "[-round float] "
            << "[-velocity|-deformation] "
@@ -368,6 +373,7 @@ int main(int argc, char**argv) {
     bool do_proj = false;
     bool do_cut  = false;
     bool do_round = false;
+    std::string reduce_to_domain = "";
     Float round_prec = sqrt(std::numeric_limits<Float>::epsilon());
     show_type show = show_none;
     // ----------------------------
@@ -475,6 +481,11 @@ int main(int argc, char**argv) {
             if (i+1 == argc || !isdigit(argv[i+1][0])) usage();
             i_comp_name = argv[++i];
         }
+        else if (strcmp (argv[i], "-domain") == 0)   {
+
+            if (i+1 == argc) usage();
+            reduce_to_domain = argv[++i];
+        }
         else if (strcmp (argv[i], "-round") == 0)   {
 
 	    do_round = true;
@@ -559,7 +570,7 @@ int main(int argc, char**argv) {
       if (!def_fill_opt)       dout.os() << nofill;
     }
     if (i_comp_name != "") {
-      // compute i_comp:
+      // compute uh[i_comp]:
       switch (uh.valued_tag()) {
         case space_constant::vector: {
           i_comp = atoi (i_comp_name.c_str());
@@ -579,13 +590,17 @@ int main(int argc, char**argv) {
       field_basic<Float,sequential> vh = uh[i_comp];
       uh = vh;
     }
+    if (reduce_to_domain != "") {
+      field_basic<Float,sequential> vh = uh[reduce_to_domain];
+      uh = vh;
+    }
     if (! do_proj && render == mayavi_render && uh.valued() != "scalar"
-        && ! uh.get_space().get_numbering().is_continuous()) {
+        && uh.get_space().get_numbering().has_compact_support_inside_element()) {
       // non-scalar vector & tensor mayavi visualization may be continuous
       warning_macro ("mayavi: perform L2-projection to continuous space");
       do_proj = true;
     }
-    if (do_proj && ! uh.get_space().get_numbering().is_continuous()) {
+    if (do_proj && uh.get_space().get_numbering().has_compact_support_inside_element()) {
       const space_basic<Float,sequential>& Uh = uh.get_space();
       size_t k = Uh.degree();
       if (k == 0) k++;
@@ -621,5 +636,24 @@ int main(int argc, char**argv) {
                << uh.max() << endl;
           return 0;
     }
+    // generates label (mayavi support it now):
+    {
+      string root_filename = iorheo::getbasename(dout.os());
+      string label = (root_filename == "output") ? "" : root_filename;
+      if (mark != "") {
+        // label = (label == "") ? mark : label + "." + mark;
+        label = mark;
+      }
+      if (i_comp_name != "") { 
+         label = (label == "") ? "value" : label;
+         label = label + "[" + i_comp_name + "]";
+      }
+      if (reduce_to_domain != "") {
+         label = (label == "") ? "value" : label;
+         label = label + "[" + reduce_to_domain + "]";
+      }
+      dout.os() << setmark (label);
+    }
+    // final output:
     dout << uh;
 }
