@@ -400,11 +400,62 @@ field_basic<T,M>::dis_evaluate (const point_basic<T>& x, size_type i_comp) const
 #endif // _RHEOLEF_HAVE_MPI
   return value;
 }
+#ifdef TO_CLEAN
 // ----------------------------------------------------------------------------
 // algebra
 // ----------------------------------------------------------------------------
 template <class T, class M>
 field_basic<T,M>
+field_basic<T,M>::operator* (const field_basic<T,M>& vh) const
+{
+  if (   get_space().valued_tag() == space_constant::scalar  &&
+      vh.get_space().valued_tag() == space_constant::scalar) {
+    field_basic<T,M> wh (get_space());
+    std::transform(begin_dof(), end_dof(), vh.begin_dof(), wh.begin_dof(), std::multiplies<T>());
+    return wh;
+  } 
+  if (get_space().valued_tag() == space_constant::scalar) {
+    // scalar*vector or scalar*tensor
+    field_basic<T,M> wh (vh.get_space());
+    for (size_type i_comp = 0, n_comp = vh.size(); i_comp < n_comp; i_comp++) {
+      std::transform(begin_dof(), end_dof(), vh[i_comp].begin_dof(), wh[i_comp].begin_dof(), std::multiplies<T>());
+    }
+    return wh;
+  }
+  if (vh.get_space().valued_tag() == space_constant::scalar) {
+    // vector*scalar or tensor*scalar
+    field_basic<T,M> wh (get_space());
+    for (size_type i_comp = 0, n_comp = size(); i_comp < n_comp; i_comp++) {
+      std::transform(operator[](i_comp).begin_dof(), operator[](i_comp).end_dof(), vh.begin_dof(), wh.begin_dof(), std::multiplies<T>());
+    }
+    return wh;
+  }
+  fatal_macro ("field*field: unexpected "<<valued() << "*" << vh.valued());
+  return field_basic<T,M>();
+}
+template <class T, class M>
+field_basic<T,M>
+field_basic<T,M>::operator/ (const field_basic<T,M>& vh) const
+{
+  if (   get_space().valued_tag() == space_constant::scalar  &&
+      vh.get_space().valued_tag() == space_constant::scalar) {
+    field_basic<T,M> wh (get_space());
+    std::transform(begin_dof(), end_dof(), vh.begin_dof(), wh.begin_dof(), std::divides<T>());
+    return wh;
+  } 
+  if (vh.get_space().valued_tag() == space_constant::scalar) {
+    // vector/scalar or tensor/scalar
+    field_basic<T,M> wh (get_space());
+    for (size_type i_comp = 0, n_comp = size(); i_comp < n_comp; i_comp++) {
+      std::transform(operator[](i_comp).begin_dof(), operator[](i_comp).end_dof(), vh.begin_dof(), wh.begin_dof(), std::divides<T>());
+    }
+    return wh;
+  }
+  fatal_macro ("field/field: unexpected "<<valued() << "/" << vh.valued());
+  return field_basic<T,M>();
+}
+template <class T, class M>
+field_basic<T,M>
 norm2 (const field_basic<T,M>& uh)
 {
   typedef typename field_basic<T,M>::size_type size_type;
@@ -431,8 +482,8 @@ norm2 (const field_basic<T,M>& uh)
         case 1 :  return sqr(uh(0,0));
         case 2 :  return sqr(uh(0,0)) + sqr(uh(1,1)) + 2*sqr(uh(0,1));
         default : return sqr(uh(0,0)) + sqr(uh(1,1)) + 2*sqr(uh(0,1))
-                       + sqr(uh(2,2))               + 2*(sqr(uh(0,2))
-                                                       + sqr(uh(1,2)));
+                       + sqr(uh(2,2))                + 2*sqr(uh(0,2))
+                                                     + 2*sqr(uh(1,2));
       }
     }
     default: {
@@ -441,6 +492,7 @@ norm2 (const field_basic<T,M>& uh)
     }
   }
 }
+#endif // TO_CLEAN
 // ----------------------------------------------------------------------------
 // tensor component access: sigma_h(i,j)
 // ----------------------------------------------------------------------------
@@ -465,8 +517,7 @@ field_basic<T,M>::operator() (size_type i_comp, size_type j_comp)
 // ----------------------------------------------------------------------------
 #define _RHEOLEF_instanciation_base(T,M) 				\
 template class field_basic<T,M>;					\
-template odiststream& operator << (odiststream&, const field_basic<T,M>&); \
-template field_basic<T,M> norm2 (const field_basic<T,M>&);
+template odiststream& operator<< (odiststream&, const field_basic<T,M>&);
 
 #ifdef TODO
 #define _RHEOLEF_instanciation(T,M) 					\
