@@ -23,8 +23,9 @@
 
 #include "rheolef/skit.h"
 #include "rheolef/space.h"
-#include "rheolef/scalar_traits.h"
+#include "rheolef/undeterminated.h"
 #include "rheolef/misc_algo.h" // copy_n
+#include "rheolef/quadrature.h"
 
 namespace rheolef {
 
@@ -35,6 +36,7 @@ template <class T, class M> class field_indirect_const;
 template <class T, class M> class field_component;
 template <class T, class M> class field_component_const;
 template <class T, class M> class field_concat_value;
+template <class T, class M> class band_basic;
 
 //<field:  
 /*Class:field
@@ -52,55 +54,85 @@ DESCRIPTION:
   domain name indexation:
   @example
         geo omega ("circle");
-        space Vh (omega, "P1");
-        Vh.block ("boundary");
-        field uh (Vh);
+        space Xh (omega, "P1");
+        Xh.block ("boundary");
+        field uh (Xh);
         uh ["boundary"] = 0;
   @end example
 
+@findex interpolate
 INTERPOLATION:
   Interpolation of a function @code{u} in a field @code{uh} with respect to 
   the interpolation writes:
   @example
         Float u (const point& x) @{ return x[0]*x[1]; @}
         ...
-        field uh = interpolate (Vh, u);
+        field uh = interpolate (Xh, u);
   @end example
 
+@findex dual
 LINEAR ALGEBRA:
-  Linear algebra, such as @code{uh+vh}, @code{uh-vh}, @code{lambda*uh + mu*vh}, ...are supported.
-  The Euclidian dot product writes simply @code{dot(uh,vh)}.
+  Linear algebra, such as @code{uh+vh}, @code{uh-vh} and @code{lambda*uh + mu*vh}, 
+  where @code{lambda} and @code{mu} are of type @code{Float}, are supported.
+  The duality product between two fields @code{lh} and @code{vh}
+  writes simply @code{dual(lh,vh)}: for discrete fields, it corresponds to a simple Euclidian dot
+  product in @code{IR^n}.
   The application of a bilinear form (@pxref{form class})
-  writes @code{m(uh,vh)} and is equivalent to @code{dot(m*uh,vh)}. 
+  writes @code{m(uh,vh)} and is equivalent to @code{dual(m*uh,vh)}. 
 
 NON-LINEAR ALGEBRA:
   Non-linear operations, such as @code{sqrt(uh)} or @code{1/uh} are also available.
   Notice that non-linear operations do not returns in general picewise polynomials:
-  the value returned by @code{sqrt(uh)} is the Lagrange interpolant of the square root of @code{uh}.
-  These operators applies directly to the set of degrees-of-freedom (dofs, see below).
+  the value returned by @code{sqrt(uh)} may be filtered by @code{interpolate},
+  @example
+        field vh = interpolate (Xh, sqrt(uh));
+  @end example
+  the Lagrange interpolant, to becomes a piecewise polynomial:
   All standard unary and binary math functions @code{abs, cos, sin}... are available
   on fields. Also @code{sqr(uh)}, the square of a field, and @code{min(uh,vh)}, @code{max(uh,vh)}
   are provided. Binary functions can be used also with a scalar, as in
   @example
-        field wh = max (abs(uh), 0);
-        field zh = pow (abs(uh), 1./3);
+        field vh = interpolate (Xh, max (abs(uh), 0));
+        field wh = interpolate (Xh, pow (abs(uh), 1./3));
   @end example
-  For applying a user-provided function to a field, use:
+  For applying a user-provided function to a field, use
+  the @code{compose} function:
 @example
-        field vh = compose(f, uh);
-        field wh = compose(f, uh, vh);
+        field vh = interpolate(Xh, compose(f, uh));
+        field wh = interpolate(Xh, compose(f, uh, vh));
 @end example
   The composition supports also general unary and binary class-functions.
+  Also, the multiplication @code{uh*vh} and the division @code{uh/vh}
+  returns a result that is not in the same discrete finite element space:
+  its result may be filtered by the @code{interpolate} operator:
+@example
+        field wh = interpolate(Xh, uh*vh);
+@end example
+  Any function or class function can be used in nonlinear expressions:
+  the function is interpolated in the specified finite element space.
 
-  For convenience, @code{uh.max()}, @code{uh.min()} and @code{uh.max_abs()} 
-  retuns respectively the maximum, minimum and maximum of the absolute value.
+@findex normal
+  There is a special predefined class-function named @code{normal} that represents the 
+  outer unnit normal vector on a boundary domain or surfacic mesh:
+@example
+        size_t k = omega.order();
+        string n_approx = "P" + itos(k-1) + "d";
+        space Nh (omega["boundary"], n_approx, "vector");
+        field nh = interpolate(Nh, normal());
+@end example
+  The normal() function could appear in any nonlinear field expression:
+  it is evaluated on the fly, based on the current mesh.
+  Notice that when using isoparametric elements, the normal vector is no more
+  constant along any face of the mesh.
+  Also, on general curved domains, the unit normal vector is discontinuous
+  accross boundary element interfaces.
 
 ACCESS BY DOMAIN:
   The restriction of a field to a geometric domain, says @code{"boundary"}
   writes @code{uh["boundary"]}: it represents the trace of the field on
   the boundary:
   @example
-        space Vh (omega, "P1");
+        space Xh (omega, "P1");
         uh["boundary"] = 0;
   @end example
   Extraction of the trace as a field is also possible:
@@ -114,8 +146,8 @@ ACCESS BY DOMAIN:
 VECTOR VALUED FIELD:
   A vector-valued field contains several components, as:
   @example
-        space Vh (omega, "P2", "vector");
-        field uh (Vh);
+        space Xh (omega, "P2", "vector");
+        field uh (Xh);
         field vh = uh[0] - uh[1];
         field nh = norm (uh);
   @end example
@@ -126,7 +158,7 @@ TENSOR VALUED FIELD:
   A tensor-valued field can be constructed and used as:
   @example
         space Th (omega, "P1d", "tensor");
-        field sigma_h (Vh);
+        field sigma_h (Xh);
         field trace_h = sigma(0,0) + sigma_h(1,1);
         field nh = norm (sigma_h);
   @end example
@@ -178,6 +210,10 @@ LOW-LEVEL DEGREE-OF-FREEDOM ACCESS:
   arrays of dofs on the current processor.
   See @pxref{array class} for more about distributed arrays.
 
+  For convenience, @code{uh.max()}, @code{uh.min()} and @code{uh.max_abs()} 
+  retuns respectively the maximum, minimum and maximum of the absolute value
+  of the degrees of freedom.
+
 FILE FORMAT:
   TODO
 
@@ -195,11 +231,14 @@ public :
 // typedefs:
 
     typedef typename std::size_t            size_type;
-    typedef T                               value_type;
-    typedef typename scalar_traits<T>::type float_type;
+    typedef M                               memory_type;
+    typedef T                               scalar_type;
+    typedef typename float_traits<T>::type  float_type;
+//  typedef undeterminated_basic<T>         value_type; // TODO
+    typedef T                               value_type; // TO_CLEAN
+    typedef space_constant::valued_type     valued_type;
     typedef geo_basic  <float_type,M>       geo_type;
     typedef space_basic<float_type,M>       space_type;
-    typedef space_constant::valued_type     valued_type;
     class iterator;
     class const_iterator;
 
@@ -296,6 +335,7 @@ public :
 
     T dis_evaluate (const point_basic<T>& x, size_type i_comp = 0) const;
     T operator()   (const point_basic<T>& x) const { return dis_evaluate (x,0); }
+    point vector_evaluate (const point_basic<T>& x) const;
 
 // internals:
 public:
@@ -307,6 +347,25 @@ public:
     // propagate changed values shared at partition boundaries to others procs
     void dis_dof_update() const;
 
+    template <class Expr>
+    void assembly_internal (
+        const geo_basic<T,M>&         dom,
+        const geo_basic<T,M>&         band,
+        const band_basic<T,M>&        gh,
+        const Expr&                   expr,
+        const quadrature_option_type& qopt,
+        bool                          is_on_band);
+    template <class Expr>
+    void assembly (
+        const geo_basic<T,M>&         domain,
+        const Expr&                   expr,
+        const quadrature_option_type& qopt);
+    template <class Expr>
+    void assembly (
+        const band_basic<T,M>&        gh,
+        const Expr&                   expr,
+        const quadrature_option_type& qopt);
+
 protected:
     void dis_dof_update_internal() const;
     void dis_dof_update_needed() const;
@@ -323,12 +382,6 @@ idiststream& operator >> (odiststream& ips, field_basic<T,M>& u);
 template <class T, class M>
 odiststream& operator << (odiststream& ops, const field_basic<T,M>& uh);
 
-template <class T, class M>
-field_basic<T,M> norm (const field_basic<T,M>& uh);
-
-template <class T, class M>
-field_basic<T,M> norm2 (const field_basic<T,M>& uh);
-
 typedef field_basic<Float> field;
 typedef field_basic<Float,sequential> field_sequential;
 //>field:
@@ -606,6 +659,7 @@ operator << (odiststream& ops, const field_basic<T,M>& uh)
 {
   return uh.put (ops);
 }
+#ifdef TO_CLEAN
 template <class T, class M>
 inline
 field_basic<T,M>
@@ -613,6 +667,7 @@ norm (const field_basic<T,M>& uh)
 {
   return sqrt(norm2(uh));
 }
+#endif // TO_CLEAN
 
 }// namespace rheolef
 # endif /* _RHEOLEF_FIELD_H */
