@@ -34,6 +34,7 @@
 #include "rheolef/dis_accumulate.h"
 #include "rheolef/dis_inner_product.h"
 #include "rheolef/pretty_name.h"
+#include "rheolef/form.h"
 
 #include <boost/mpl/bool.hpp>
 #include <boost/proto/core.hpp>
@@ -52,6 +53,10 @@ using proto::_;
 
 // forward declaration:
 template <class Expr> struct field_expr;
+template <class Expr> class field_nonlinear_expr;
+template <class T, class M, class VfTag> class test_basic;
+template <class RawExpr, class VfTag> class field_vf_expr;
+template <class RawExpr> class form_vf_expr;
 
 // -----------------------------------------------------------
 // iterator:
@@ -82,7 +87,8 @@ struct field_begin : proto::callable {
     }
 };
 // Here is the grammar that replaces field terminals with their begin iterators
-struct field_grammar_begin : proto::or_<
+struct field_grammar_begin
+  : proto::or_<
         proto::when <proto::terminal<field_basic<_, _> >,                field_begin(proto::_value)>
       , proto::when <proto::terminal<field_indirect<_, _> >,             field_begin(proto::_value)>
       , proto::when <proto::terminal<field_indirect_const<_, _> >,       field_begin(proto::_value)>
@@ -297,13 +303,58 @@ struct field_assign_operators_cases {
 };
 // An expression conforms to the field_grammar if it is a terminal or some
 // op that is not an assignment op. (Assignment will be handled specially.)
-struct field_grammar
-  : proto::or_<
+// expr : +expr | -expr | expr + expr | expr - expr
+//      | c*expr | expr*c | expr/c
+//      | f(expr) | f(expr,expr)
+//      | += expr | -= expr
+struct field_terminal :
+          proto::or_<
+            proto::terminal<field_basic<_, _> >
+           ,proto::terminal<field_indirect<_, _> >
+           ,proto::terminal<field_indirect_const<_, _> >
+           ,proto::terminal<field_component<_, _> >
+           ,proto::terminal<field_component_const<_, _> >
+          >
+{};
+
+struct field_constant_expr :
+        proto::and_<
         proto::terminal<_>
-      , proto::and_<
-            proto::nary_expr<_, proto::vararg<field_grammar> >
-          , proto::not_<field_assign_operators>
+       ,proto::not_<
+          proto::or_< // "proto::or" limited to 6 or 8 args
+            proto::or_<
+              proto::terminal<field_basic<_, _> >
+             ,proto::terminal<field_indirect<_, _> >
+             ,proto::terminal<field_indirect_const<_, _> >
+             ,proto::terminal<field_component<_, _> >
+             ,proto::terminal<field_component_const<_, _> >
+            >
+           ,proto::or_<
+              proto::terminal<form_basic<_, _> >
+             ,proto::terminal<field_nonlinear_expr<_> >
+             ,proto::terminal<test_basic<_,_,_> >
+             ,proto::terminal<field_vf_expr<_,_> >
+             ,proto::terminal<form_vf_expr<_> >
+            >
+          >
         >
+      >
+{};
+struct field_grammar
+  : proto::or_< // "proto::or" limited to 6 or 8 args
+      proto::or_<
+        field_terminal
+       ,field_constant_expr
+       ,proto::unary_plus <field_grammar>	// -expr
+       ,proto::negate     <field_grammar>	// +expr
+       ,proto::plus       <field_grammar, field_grammar>
+       ,proto::minus      <field_grammar, field_grammar>
+      >
+     ,proto::or_<
+        proto::multiplies <field_constant_expr, field_grammar> // as 2*uh
+       ,proto::multiplies <field_grammar, field_constant_expr> // as uh*2
+       ,proto::divides    <field_grammar, field_constant_expr> // as uh/2
+      >
     >
 {};
 
@@ -321,6 +372,7 @@ struct field_expr : proto::extends<Expr, field_expr<Expr>, field_domain> {
 
     typedef field_get_size_context::size_type size_type;
     typedef typename boost::result_of<field_grammar_begin(const Expr&)>::type raw_iterator;
+    typedef rheo_default_memory_model memory_type; // TODO: compute it at compile time
 
 // allocators:
 
@@ -383,21 +435,22 @@ struct field_expr : proto::extends<Expr, field_expr<Expr>, field_domain> {
 
 // random access:
     typedef typename proto::result_of::eval<const Expr, const field_subscript_context>::type value_type;
-    value_type dof (size_type i) const {
+    typedef value_type scalar_type;
+    scalar_type dof (size_type i) const {
         const field_subscript_context get_subscript(i);
         return proto::eval(*this, get_subscript);
     }
 
 #ifdef TODO
-    value_type max () const {
-      value_type val = std::numeric_limits<value_type>::min();
+    scalar_type max () const {
+      scalar_type val = std::numeric_limits<scalar_type>::min();
       for (const_iterator iter = begin_dof(), last = end_dof(); iter != last; iter++) {
         val = std::max(val, *iter);
       }
       return val;
     }
-    value_type max_abs () const { return abs(*this).max(); }
-    value_type min_abs () const { return abs(*this).min(); }
+    scalar_type max_abs () const { return abs(*this).max(); }
+    scalar_type min_abs () const { return abs(*this).min(); }
 #endif // TODO
 private:
     // hide this:
@@ -407,7 +460,7 @@ template <class Expr>
 inline
 odiststream& operator<< (odiststream& ops, const field_expr<Expr>& uh)
 {
-    typedef typename field_expr<Expr>::value_type T;
+    typedef typename field_expr<Expr>::scalar_type T;
     field_basic<T> tmp = uh; // TODO: add M as <T,M>
     return tmp.put (ops);
 }
@@ -499,6 +552,17 @@ field_basic<T,M>::field_basic (const field_expr<Expr>& expr)
 {
     operator= (expr);
 }
+// ---------------------------------------------------------------------------
+// TODO: form := diag(field_expr) => how to deduce M from field_expr<Expr> ?
+// ---------------------------------------------------------------------------
+template<class Expr>
+inline
+form_basic<typename field_expr<Expr>::scalar_type, rheo_default_memory_model>
+diag (const field_expr<Expr>& expr)
+{
+  typedef typename field_expr<Expr>::scalar_type T;
+  return diag(field_basic<T,rheo_default_memory_model>(expr));
+}
 
 } // namespace rheolef
 #endif // _RHEOLEF_FIELD_EXPR_H
