@@ -131,8 +131,7 @@ template <class T, class M>
 typename geo_domain_indirect_base_rep<T,M>::size_type
 geo_domain_indirect_base_rep<T,M>::dis_inod2dis_iv (size_type dis_inod) const
 {
-  error_macro ("domain.dis_inod2dis_iv: not yet");
-  return 0;
+  return _omega.dis_inod2dis_iv (dis_inod);
 }
 template <class T, class M>
 void
@@ -216,6 +215,19 @@ geo_domain_indirect_base_rep<T,M>::trace_move (
 {
   error_macro ("domain.trace_move: no implemented");
 }
+template <class T, class M>
+typename geo_domain_indirect_base_rep<T,M>::size_type
+geo_domain_indirect_base_rep<T,M>::neighbour (size_type ie, size_type loc_iside) const
+{
+  error_macro ("domain.neighbour: no implemented");
+  return std::numeric_limits<size_type>::max();
+}
+template <class T, class M>
+void
+geo_domain_indirect_base_rep<T,M>::neighbour_guard() const
+{
+  error_macro ("domain.neighbour_guard: no implemented");
+}
 // ----------------------------------------------------------------------------
 // instanciation in library
 // ----------------------------------------------------------------------------
