@@ -213,8 +213,8 @@ template <class T, class M>
 void
 space_scalar_constitution_rep<T,M>::do_act (const space_act& act)
 {
-  check_macro (get_numbering().is_continuous(),
-    "try to [un]block domain `" << act.get_domain_name() << "' on discontinuous space("
+  check_macro (!get_numbering().has_compact_support_inside_element(),
+    "try to [un]block domain `" << act.get_domain_name() << "' on discontinuous or bubble space("
     << get_numbering().name()<<"(" <<_omega.name()<<"))");
   _acts.push_back (act);
 }
@@ -439,6 +439,28 @@ space_constitution_rep<T,M>::set_ios_permutations (
 // used by field & form_assembly
 // ----------------------------------------------------------------------------
 template <class T, class M>
+typename space_constitution_rep<T,M>::size_type
+space_constitution_rep<T,M>::loc_ndof (const reference_element& hat_K) const
+{
+  switch (valued_tag()) {
+    case space_constant::scalar: {
+      return get_numbering().get_basis().size (hat_K);
+    }
+    case space_constant::vector:
+    case space_constant::tensor: {
+      // LIMITATION: assume homogeneous vector or tensor valued
+      // multi-component case: shift from i_comp=0
+      size_type loc_comp_ndof = get_numbering().get_basis().size (hat_K);
+      size_type n_comp        = size();
+      return n_comp*loc_comp_ndof;
+    }
+    default: {
+      fatal_macro ("dis_idof: hybrid multi-component: not yet");
+      return 0; // not reached
+    }
+  }
+}
+template <class T, class M>
 void
 space_constitution_rep<T,M>::dis_idof (const geo_element& K, std::vector<size_type>& dis_idof_t) const
 {
@@ -535,7 +557,7 @@ space_constitution_rep<T,M>::init_start_by_component ()
 // ---------------------------------------------------------------,--------
 template <class T, class M>
 void
-space_constitution_rep<T,M>::append_external_dof (
+space_constitution_rep<T,M>::continuous_append_external_dof (
         const geo_basic<T,M>&         dom,
         std::set<size_type>&          ext_dof_set,
         const distributor&            dof_ownership,
@@ -543,7 +565,7 @@ space_constitution_rep<T,M>::append_external_dof (
 {
   // assume a scalar (non-multi valued) case:
   distributor comp_ownership = ownership();
-  size_type comp_dis_ndof       = comp_ownership.dis_size();
+  size_type comp_dis_ndof    = comp_ownership.dis_size();
   std::vector<size_type> comp_dis_idof1;
   for (size_type ie = 0, ne = dom.size(); ie < ne; ie++) {
     const geo_element& K = dom [ie];
@@ -569,6 +591,60 @@ space_constitution_rep<T,M>::append_external_dof (
 }
 template <class T, class M>
 void
+space_constitution_rep<T,M>::discontinuous_append_external_dof (
+        const geo_basic<T,M>&         dom,
+        std::set<size_type>&          ext_dof_set,
+        const distributor&            dof_ownership,
+        const distributor&            start_by_component) const
+{
+  size_type dim     = dom.dimension();
+  size_type map_dim = dom.map_dimension();
+  if (dim > map_dim) {
+    // 3d surfacic mesh or 2d lineique mesh are not yet treated for the DG method
+    // TODO: compute neighbours for such meshes
+    return;
+  }
+  if (dom.name() != dom.get_background_geo().name()) {
+    // e.g. geo-domain, as omega["east"] in transmission.cc, are not yet treated for the DG method
+    // TODO: treat also geo-domains
+    return;
+  }
+  // assume a scalar (non-multi valued) case:
+  dom.neighbour_guard(); // uses S.master(i), a neighbour connnectivity
+  if (map_dim == 0) return; // no sides
+  size_type sid_dim = map_dim-1;
+  distributor comp_ownership = ownership();
+  size_type comp_dis_ndof    = comp_ownership.dis_size();
+  std::vector<size_type> comp_dis_idof1;
+  for (size_type isid = 0, nsid = dom.size(sid_dim); isid < nsid; isid++) {
+    const geo_element& S = dom.get_geo_element (sid_dim, isid);
+    for (size_type i_master = 0; i_master < 2; i_master++) {
+      size_type dis_ie = S.master(i_master); // two possible Ki such that S is a side of Ki
+      if (dis_ie == std::numeric_limits<size_type>::max()) continue; // S is on the boundary
+      const geo_element& K = dom.dis_get_geo_element (map_dim, dis_ie);
+      dis_idof (K, comp_dis_idof1);
+      for (size_type loc_idof = 0, loc_ndof = comp_dis_idof1.size(); loc_idof < loc_ndof; loc_idof++) {
+        size_type comp_dis_idof = comp_dis_idof1 [loc_idof];
+        assert_macro (comp_dis_idof < comp_dis_ndof, "idof " << comp_dis_idof1 [loc_idof] << " out of range[0:"<< comp_dis_ndof << "[");
+        if (! comp_ownership.is_owned (comp_dis_idof)) {
+          size_type iproc = comp_ownership.find_owner (comp_dis_idof);
+          size_type comp_first_dis_idof = comp_ownership.first_index(iproc);
+          assert_macro (comp_dis_idof >= comp_first_dis_idof, "unexpected comp_dis_idof");
+          size_type comp_idof = comp_dis_idof - comp_first_dis_idof;
+          size_type comp_start_idof = start_by_component.size(iproc);
+          size_type idof = comp_start_idof + comp_idof;
+          size_type first_dis_idof = dof_ownership.first_index(iproc);
+          size_type dis_idof = first_dis_idof + idof;
+          assert_macro (! dof_ownership.is_owned (dis_idof), "unexpected dis_idof="<<dis_idof<<" in range ["
+		<< first_dis_idof << ":" << dof_ownership.last_index() << "[");
+          ext_dof_set.insert (dis_idof);
+        }
+      }
+    }
+  }
+}
+template <class T, class M>
+void
 space_constitution_rep<T,M>::compute_external_dofs (
         std::set<size_type>&            ext_dof_set,
         const distributor&              dof_ownership,
@@ -576,15 +652,18 @@ space_constitution_rep<T,M>::compute_external_dofs (
 	size_type&                      i_comp) const
 {
   if (! is_hierarchical()) {
-    // loop on the main mesh
     const space_scalar_constitution<T,M>& scalar_constit = get_scalar();
-    append_external_dof (scalar_constit.get_geo(), ext_dof_set, dof_ownership, start_by_component [i_comp]);
-    // loop on all domains and insert also external dofs
-    if (scalar_constit.get_numbering().is_continuous()) {
+    if (! scalar_constit.get_numbering().has_compact_support_inside_element()) {
+      // C0 FEM: add idofs at the partition boundaries
+      continuous_append_external_dof (scalar_constit.get_geo(), ext_dof_set, dof_ownership, start_by_component [i_comp]);
+      // loop on all domains and insert also external dofs
       for (size_type idom = 0, ndom = scalar_constit.get_geo().n_domain(); idom < ndom; idom++) {
         const geo_basic<T,M>& dom = scalar_constit.get_geo().get_domain (idom);
-        append_external_dof (dom, ext_dof_set, dof_ownership, start_by_component [i_comp]);
+        continuous_append_external_dof (dom, ext_dof_set, dof_ownership, start_by_component [i_comp]);
       }
+    } else {
+      // discontinuous Galerkin: add idofs at the partition boundaries via sides
+      discontinuous_append_external_dof (scalar_constit.get_geo(), ext_dof_set, dof_ownership, start_by_component [i_comp]);
     }
     i_comp++;
     return;
